package cz.bilysklep.trainings.basicjava.swingexample;

import java.awt.BorderLayout;
import java.awt.EventQueue;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.util.List;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingWorker;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

public class SwingWorkerExampleMainWindow extends JFrame {

    private final JTextArea textArea;
    private final JProgressBar progressBar;
    private final Action startMyOperationAction;
    
    public SwingWorkerExampleMainWindow() throws HeadlessException {
        
        super("SwingWorker example");
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        textArea = new JTextArea(10, 40);
        add(new JScrollPane(textArea));
        
        progressBar = new JProgressBar(0, 100);
        add(progressBar, BorderLayout.SOUTH);
        
        JToolBar toolBar = new JToolBar();
        add(toolBar, BorderLayout.NORTH);
        
        startMyOperationAction = new StartMyOperationAction();
        
        toolBar.add(startMyOperationAction);
        
        JMenuBar menuBar = new JMenuBar();
        setJMenuBar(menuBar);
        
        JMenu fileMenu = new JMenu("File");
        menuBar.add(fileMenu);
        fileMenu.add(startMyOperationAction);
        
        pack();
    }
   
    private class StartMyOperationAction extends AbstractAction {

        public StartMyOperationAction() {
            super("Start", new ImageIcon(
                    SwingWorkerExampleMainWindow.class.getResource("operation-20.png")));
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK));
            putValue(SHORT_DESCRIPTION, "Starts my operation");
        }
        
        @Override
        public void actionPerformed(ActionEvent e) {
            new MyOperationWorker().execute();
        }
        
    }
    
    private class MyOperationWorker extends SwingWorker<Long, Integer> {

        public MyOperationWorker() {
            assert EventQueue.isDispatchThread();
            startMyOperationAction.setEnabled(false);
            addPropertyChangeListener(e -> {
                if (e.getPropertyName().equals("progress")) {
                    progressBar.setValue((int) e.getNewValue());
                }
            });
        }
        
        @Override
        protected Long doInBackground() throws Exception {
            long sum = 0;
            for(int i = 0; i < 100; i++) {
                setProgress(i);
                sum += i;
                makeStep();
                if (i % 5 == 0) {
                    publish(i);
                }
            }
            setProgress(100);
            return sum;
        }

        @Override
        protected void process(List<Integer> steps) {
            for (Integer step : steps) {
                textArea.append("Step: " + step + "\n");
            }
        }

        @Override
        protected void done() {
            try {
                long sum = get();
                textArea.append("Result is: " + sum + "\n");
            } catch (InterruptedException ex) {
                throw new AssertionError(ex);
            } catch (ExecutionException ex) {
                JOptionPane.showMessageDialog(SwingWorkerExampleMainWindow.this,
                        ex.getCause().getLocalizedMessage(), 
                    "Error when performing MyOperation", JOptionPane.ERROR_MESSAGE);
           } finally {
                startMyOperationAction.setEnabled(true);
           }
        }
        
    }
    
    private void makeStep() {
        try {
            Thread.sleep(100);
        } catch (InterruptedException ex) {
            throw (CancellationException) new CancellationException("Operation interrupted").initCause(ex);
        }
    }
    
    
    private static void initNimbusLookAndFeel() {
        try {
            for (UIManager.LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException | InstantiationException | 
                IllegalAccessException | UnsupportedLookAndFeelException ex) {
            Logger.getLogger(SwingWorkerExampleMainWindow.class.getName()).log(
                    Level.SEVERE, "Nimbus layout initialization failed", ex);
        }
        
    }
    
    public static void main(String[] args) {
        
        initNimbusLookAndFeel();
        
        EventQueue.invokeLater(() -> {
            new SwingWorkerExampleMainWindow().setVisible(true);
        });

    }
    
}
